# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Experiment'
        db.create_table(u'variant_experiment', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=50)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('variants', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'variant', ['Experiment'])


    def backwards(self, orm):
        # Deleting model 'Experiment'
        db.delete_table(u'variant_experiment')


    models = {
        u'variant.experiment': {
            'Meta': {'object_name': 'Experiment'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'variants': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['variant']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
import six

from .utils import get_experiment_cookie_name


class VariantMiddleware(object):
    def process_request(self, request):
        request.variant_experiments = {}

    def process_response(self, request, response):
        for name, variant in six.iteritems(request.variant_experiments):
            if variant:
                cookie_name = get_experiment_cookie_name(name)
                response.set_cookie(
                    cookie_name, variant,
                    max_age=getattr(
                        settings, 'VARIANT_MAX_COOKIE_AGE', 2592000),
                    secure=getattr(settings, 'VARIANT_SECURE_COOKIE', False))

        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.text import slugify

from .models import Experiment


def get_experiment_variant(request, experiment_name):
    try:
        if experiment_name in request.variant_experiments:
            return request.variant_experiments[experiment_name]
    except AttributeError:
        raise ImproperlyConfigured(
            'VariantMiddleware must be enabled to use Variant experiments.')

    try:
        experiment = Experiment.objects.get(name=experiment_name, active=True)
    except Experiment.DoesNotExist:
        experiment = None
        variant = None

    if experiment:
        cookie_name = get_experiment_cookie_name(experiment_name)
        variant = request.COOKIES.get(cookie_name, None)

        if not variant or variant not in experiment.get_variants():
            variant = experiment.choose_variant()

    request.variant_experiments[experiment_name] = variant
    return variant


def get_experiment_cookie_name(experiment_name):
    cookie_pattern = getattr(settings, 'VARIANT_EXPERIMENT_COOKIE', 'dvc_{}')
    return slugify(cookie_pattern.format(experiment_name))



#!/usr/bin/env python

from setuptools import setup

setup(
    name='django-variant',
    version='0.0.2',
    description='Django variant testing framework',
    author='Jeremy Sattefield',
    author_email='jsatt22@gmail.com',
    url='https://github.com/jsatt/django-variant',
    #license='',
    packages=[
        'variant', 'variant.migrations'],
    install_requires=[
        'Django>=1.5', 'six'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Site Management'],
)
